/* This file is part of Lcrzo.
*/

#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
typedef PCHAR (WINAPI*pPacket_PacketGetVersion) (void);

/*-------------------------------------------------------------*/
int lcrzo_priv_winpcap_version2(lcrzo_string *pstrver);
int lcrzo_priv_winpcap_version2(lcrzo_string *pstrver)
{
  HINSTANCE hinstance;
  pPacket_PacketGetVersion pf;
  int ret, ret2;

  /* load the DLL */
  hinstance = LoadLibrary("packet.dll");
  if (hinstance == NULL) {
    return (LCRZO_ERR_FULOADLIBRARY);
  }

  /* init function */
  pf = (pPacket_PacketGetVersion) GetProcAddress(hinstance, "PacketGetVersion");
  if (pf == NULL) {
    FreeLibrary(hinstance);
    return (LCRZO_ERR_FUGETPROCADDRESS);
  }

  /* call function */
  ret = lcrzo_string_initm_text((*pf)(), pstrver);

  /* unload the DLL */
  ret2 = FreeLibrary(hinstance);
  if (!ret2)
    return (LCRZO_ERR_FUFREELIBRARY);

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_winpcap_version(lcrzo_uint32 *pdrivermajor,
                               lcrzo_uint32 *pdriverminor)
{
  static lcrzo_uint32 drivermajor = 0xFFFFFFFFu;
  static lcrzo_uint32 driverminor = 0xFFFFFFFFu;
  PCHAR pstrver, pc;
  lcrzo_uint32 total;
  lcrzo_bool majorset;
  char c;
  int ret;

  /* if already known, return directly */
  if (drivermajor != 0xFFFFFFFFu && driverminor != 0xFFFFFFFFu) {
    if (pdrivermajor!=NULL) *pdrivermajor = drivermajor;
    if (pdriverminor!=NULL) *pdriverminor = driverminor;
    return(LCRZO_ERR_OK);
  }

  ret = lcrzo_priv_winpcap_version2(&pstrver);
  if (ret != LCRZO_ERR_OK) {
    /* there is no other way to detect version, so say 0.0 */
    drivermajor = 0;
    driverminor = 0;
    if (pdrivermajor!=NULL) *pdrivermajor = drivermajor;
    if (pdriverminor!=NULL) *pdriverminor = driverminor;
    return(LCRZO_ERR_OK);
  }

  /* obtain version */
  /* accept :
       "3.*"       ->  3  and  0
       "3\.3.*"    ->  3  and  3
       "33\.33.*"  ->  33 and 33
       "33\.33a.*"  -> 33 and 33
       etc.
  */
  pc = pstrver;
  total = 0;
  drivermajor = 0;
  driverminor = 0;
  majorset = LCRZO_FALSE;
  while (1) {
    c = *pc;
    if (isdigit(c)) {
      /* ignore the case of an big version number which overflows */
      total *= 10;
      total += c - '0';
    } else if (c == '.' || c == ' ') {
      if (!majorset) {
        drivermajor = total;
        majorset = LCRZO_TRUE;
        total = 0;
      } else {
        driverminor = total;
        break;
      }
    } else {
      if (!majorset) {
        drivermajor = total;
      } else {
        driverminor = total;
      }
      break;
    }
    pc++;
  }
  lcrzo_er(lcrzo_string_free2(&pstrver));
  
  /* set output */
  if (pdrivermajor!=NULL) *pdrivermajor = drivermajor;
  if (pdriverminor!=NULL) *pdriverminor = driverminor;
  return(LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
